BACKGROUND_MESSAGE_SUBJECT = "__BACKGROUND_MESSAGE__"
CHAT_SUBJECT = "__CHAT__"
TOOL_CHANGE_SUBJECT = "__TOOL_CALL__"
AGENT_FUNCTION_CALL_SUBJECT = "__AGENT_FUNCTION_CALL__"
ERROR_SUBJECT = "__ERROR__"
EVENT_SUBJECT = "__EVENT__"
NOTIFICATION_SUBJECT = "__NOTIFICATION__"
COMMAND_SUBJECT = "__COMMAND__"
SYSTEM_MESSAGE_SUBJECT = "__SYSTEM_MESSAGE__"
LOG_SUBJECT = "__LOG__"
CONFIG_SUBJECT = "__CONFIG__"

GLOBAL_CHANNEL = "__GLOBAL_CHANNEL__"
GLOBAL_SESSION_ID = "__GLOBAL__"
GLOBAL_EVENT_CHANNEL = "__GLOBAL_EVENT_CHANNEL__"
GLOBAL_NOTIFICATION_CHANNEL = "__GLOBAL_NOTIFICATION_CHANNEL__"


BACKGROUND_SERVICE_NAME = "background.service"
CONFIG_SERVICE_NAME = "config.service"
LOGGER_SERVICE_NAME = "logger.service"
NOTIFICATION_SERVICE_NAME = "notification.service"
OCR_SERVICE_NAME = "ocr.service"
VM_EVENT_MONITOR_SERVICE_NAME = "vm_event_monitoring.service"
REGISTRY_SERVICE_NAME = "registry.service"
